#!/bin/sh
# $Id: doinstall.sh,v 6.3 2001/06/23 09:05:09 cher Exp $ 

# Copyright (C) 1999-2001 Alexander Chernov <cher@ispras.ru>

#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

prefix_file=
[ -x ./etc/env_prefix.sh ] && prefix_file=./etc/env_prefix.sh

if [ x"${PROJECT_ENV_PREFIX}" = x ]
then
    if [ x"${prefix_file}" = x ]
    then
        echo "Cannot guess project environment prefix" >&2
        exit 1
    fi
    . ${prefix_file}
    if [ x"${PROJECT_ENV_PREFIX}" = x ]
    then
        echo "Cannot guess project environment prefix" >&2
        exit 1
    fi
fi

prj_home_name=${PROJECT_ENV_PREFIX}_HOME
prj_config_name=${PROJECT_ENV_PREFIX}_CONFIG

eval 'prj_home=$'${prj_home_name}
eval 'prj_config=$'${prj_config_name}

# Guess prj_home
if [ x"${prj_home}" = x ]
then
  # assumed, that the utility is started from the root dir
  prj_home=`pwd`
  eval ${prj_home_name}="${prj_home}"
  export ${prj_home_name}
fi

# Guess prj_config
if [ x"${prj_config}" = x ]
then
  if prj_config=`./etc/config.guess`
  then
    :
  else
    echo "Cannot guess the canonic configuration name" >&2
    exit 1
  fi

  eval ${prj_config_name}="${prj_config}"
  export ${prj_config_name}
fi

echo "${prj_home_name}=${prj_home}"
echo "${prj_config_name}=${prj_config}"

# Load configuration file
cfgfile="${prj_home}/etc/.cfg-${prj_config}/compile.sh"
if [ -f "${cfgfile}" ]
then
  . "${cfgfile}"
else
  echo "Configuration files do not exist. Please, run configure." >&2
  exit 1
fi

eval prefix="${CONF_PREFIX}"
echo "PREFIX=${prefix}"

if [ x"${prefix}" = x"${prj_home}" ]
then
  echo "PREFIX equals to build home. No installation necessary." >&2
  exit 0
fi

set -e
if [ -d "${prefix}" ]; then :
else
  mkdir "${prefix}"
fi

if [ -d "${prefix}"/etc ]; then :
else
  mkdir "${prefix}"/etc
fi

cfgsubdir="etc/.cfg-${prj_config}"
if [ -d "${prefix}/${cfgsubdir}" ]; then :
else
  mkdir "${prefix}/${cfgsubdir}"
fi

echo "${cfgsubdir}/runtime.sh"
cp -fp "${prj_home}/${cfgsubdir}/runtime.sh" "${prefix}/${cfgsubdir}"
echo "etc/config.guess"
cp -fp "${prj_home}/etc/config.guess" "${prefix}/etc"
echo "etc/version.sh"
cp -fp "${prj_home}/etc/version.sh" "${prefix}/etc"

tmptar="${prefix}/install.tar"

# Copy bin directory
tar cf "${tmptar}" bin
( cd "${prefix}"; tar xvf "${tmptar}"; )
rm "${tmptar}"

if [ -d "${prj_home}/libexec/${prj_config}" ]
then
  tar cf "${tmptar}" "libexec/${prj_config}"
  ( cd "${prefix}"; tar xvf "${tmptar}"; )
  rm "${tmptar}"
elif [ x"${CONFIG_RUNTIME}" != x -a -d "${prj_home}/libexec/${CONFIG_RUNTIME}" ]
then
  tar cf "${tmptar}" "libexec/${CONFIG_RUNTIME}"
  ( cd "${prefix}"; tar xvf "${tmptar}"; )
  rm "${tmptar}"
fi

if [ -d "${prj_home}/share" ]
then
  tar cf "${tmptar}" share
  ( cd "${prefix}"; tar xvf "${tmptar}"; )
  rm "${tmptar}"
fi

exit 0
