#!/bin/sh
# $Id: mkver.sh,v 6.6 2001/06/23 07:09:56 cher Exp $

# Copyright (C) 1999-2001 Alexander Chernov <cher@ispras.ru>

#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

# if AWK environment variable not set, set it to `awk'
[ x"${AWK}" = x ] && AWK=awk

filename=$1
patchlevelnum=$2
if [ x"${filename}" = x ]
then
  echo "output file name expected" >&2
  exit 1
fi

. ./version.sh

revident=
if [ x"${patchlevelnum}" != x -a -f "${patchlevelnum}" ]
then
  revident=`cat "${patchlevelnum}"`
fi

echo "Version number: ${REUSE_VERSION}"
echo "Revision identification: ${revident}"

dbfile=etc/versions.db

# Search the version database for this version
if [ -f ${dbfile} ]
then
  patchlevel=`${AWK} -v K=${revident} -v V=${REUSE_VERSION} '$1 == K && $2 == V { print $3; exit 0; }' ${dbfile}`
else
  patchlevel=
fi

if [ x"${patchlevel}" = x ]
then
  # Search for the last patchlevel
  if [ -f ${dbfile} ]
  then
    lastlevel=`${AWK} -v V=${REUSE_VERSION} '$2 == V { R=$3; } END { print R; }' ${dbfile}`
    if [ x"${lastlevel}" = x ]
    then
      patchlevel=0
    else
      patchlevel=`expr "${lastlevel}" + 1`
    fi
  else
    patchlevel=0
  fi

  if [ ! -w version.sh ]
  then
    # update version database 
    if [ -f ${dbfile} ]
    then
      chmod +w ${dbfile}
    else
      echo '# $''Id''$' > ${dbfile}
      echo '# Version database. Updated automatically.' >> ${dbfile}
    fi

    echo "${revident} ${REUSE_VERSION} ${patchlevel}" >> ${dbfile}
  else
    patchlevel="${patchlevel}_Dev"
  fi

  if [ -d etc/CVS -a ! -w version.sh ]
  then
    (cd etc; cvs ci -m "" `basename ${dbfile}`; )
  fi
fi

echo "Patchlevel: ${patchlevel}"

cat <<EOF > ${filename}.c
#include <p_config.h>
#include CONF_DEPFILE("reuse_version.h")

char const reuse_release_str[]=
"$" "Revision: REUSE-${REUSE_VERSION}.${patchlevel} (${revident}) " "$";
EOF

cat <<EOF > ${filename}.h
#ifndef REUSE_VERSION
#define REUSE_VERSION ${REUSE_VERSION}

extern char const reuse_release_str[];
static char const * const reuse_release = reuse_release_str;

#endif
EOF

exit 0
