/* -*- mode:c -*- */
/* $Id: fsinsfs.c,v 1.1 2002/05/19 11:17:18 cher Exp $ */

/* Copyright (C) 2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include "lconfig.h"
#include <p_config.h>

#include "reuse/FlexString.h"
#include "reuse/ReuseDefs.h"
#include "reuse/xalloc.h"

#include <assert.h>

#define CLUSTER           32
#define ALIGN(size)       ((size+31) & 0xffffffe0)

#ifdef REUSE_DEBUG
#  define CHECK(ptr)        assert((ptr) != NULL)
#  define CHECKPFS(pfs)     assert((pfs)->Used == 0 || (pfs)->Used < (pfs)->Allocd)
#else
#  define CHECK(p)
#  define CHECKPFS(p)
#endif /* REUSE_DEBUG */

void reuse_fs_range_error(char const*);

void
_fsInsFS(tFString *pfsDest,tFString *pfsSrc,int pos)
{
  CHECKPFS(pfsDest);
  CHECKPFS(pfsSrc);

#ifdef REUSE_DEBUG
  if (pfsDest == pfsSrc)
    {
      fputs ("_fsInsFS: Dest and Src strings are the same!\n", stderr);
      abort();
    }
#endif

  if(pos < 0 || (unsigned int) pos > pfsDest->Used)
    reuse_fs_range_error("_fsInsFS");
  
  if(pfsSrc->Used == 0) return;
  if(pfsDest->Used + pfsSrc->Used >= pfsDest->Allocd)
    {
      pfsDest->String =
        (char*) xrealloc(pfsDest->String,
                        pfsDest->Allocd = ALIGN(pfsDest->Used+pfsSrc->Used+1));
      pfsDest->String[pfsDest->Used] = 0;
      CHECK(pfsDest->String);
    }
  memmove(pfsDest->String+pos+pfsSrc->Used,
          pfsDest->String+pos,
          pfsDest->Used-pos+1);
  memcpy(pfsDest->String+pos,pfsSrc->String,pfsSrc->Used);
  pfsDest->Used += pfsSrc->Used;
}

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "tFString")
 * End:
 */
