/* -*- mode:c -*- */
/* $Id: fsinsstr.c,v 1.1 2002/05/19 11:17:18 cher Exp $ */

/* Copyright (C) 2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include "lconfig.h"
#include <p_config.h>

#include "reuse/FlexString.h"
#include "reuse/ReuseDefs.h"
#include "reuse/xalloc.h"

#include <assert.h>

#define CLUSTER           32
#define ALIGN(size)       ((size+31) & 0xffffffe0)

#ifdef REUSE_DEBUG
#  define CHECK(ptr)        assert((ptr) != NULL)
#  define CHECKPFS(pfs)     assert((pfs)->Used == 0 || (pfs)->Used < (pfs)->Allocd)
#else
#  define CHECK(p)
#  define CHECKPFS(p)
#endif /* REUSE_DEBUG */

void reuse_fs_range_error(char const *);

void
_fsInsStr(tFString *pfs,char *str,int pos)
{
  int len;
  
  CHECKPFS(pfs);
  if(pos < 0 || (unsigned int) pos > pfs->Used)
    reuse_fs_range_error("_fsInsStr");
  
  if(str == NULL) return;
  len = strlen(str);
  if(len == 0) return;
  if(pfs->Used+len >= pfs->Allocd)
    {
      pfs->String = (char*)
        xrealloc(pfs->String, pfs->Allocd = ALIGN(pfs->Used+len+1));
      pfs->String[pfs->Used] = 0;
      CHECK(pfs->String);
    }
  memmove(pfs->String+pos+len,pfs->String+pos,pfs->Used-pos+1);
  memcpy(pfs->String+pos,str,len);
  pfs->Used += len;
}

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "tFString")
 * End:
 */
