/* $Id: memfile.c,v 6.6 2002/02/21 04:56:00 cher Exp $ */

/* Copyright (C) 1999-2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

/**
 * FILE:    utils/memfile.c
 * PURPOSE: virtual file in memory
 */

static char const cvsid[]=
"$Id: memfile.c,v 6.6 2002/02/21 04:56:00 cher Exp $";

#include "lconfig.h"
#include <p_config.h>

#include CONF_DEPFILE("reuse_version.h")

#include "reuse/memfile.h"
#include "reuse/xalloc.h"

#include <stdio.h>
#include <string.h>

/* The size of the allocation cluster */
#define CLUSTER 32768

struct chunk_s
{
  struct chunk_s *next;
  char            buf[CLUSTER];
};

struct memfile_s
{
  struct chunk_s *fp;           /* first chunk */
  struct chunk_s *wp;           /* current write chunk */
  struct chunk_s *rp;           /* current read chunk */
  unsigned int    wo;           /* write offset in the chunk */
  unsigned int    ro;           /* read offset in the chunk */
};

  MEMFILE *
memfile_new()
{
  MEMFILE *f = 0;

  f = xcalloc(1, sizeof(*f));
  return f;
}

  MEMFILE *
memfile_delete(MEMFILE *f)
{
  struct chunk_s *p, *v;

  if (!f) return NULL;
  for (p = f->fp; p; p = v) {
    v = p->next;
    xfree(p);
  }
  xfree(f);
  return NULL;
}

  unsigned long int
memfile_write(MEMFILE *f, void *vbuf, unsigned long int size)
{
  unsigned int    osz = size;
  unsigned char  *buf = (unsigned char *) vbuf;
  unsigned char  *ptr = (unsigned char *) vbuf;
  unsigned int    ws  = 0;
  struct chunk_s *nc  = 0;

  if (!size || !f || !buf) return 0;

  while (1) {
    /* append to the chunk */
    if (f->wp && f->wo < CLUSTER) {
      ws = CLUSTER - f->wo;
      if (ws > size) ws = size;
      memcpy(f->wp->buf + f->wo, ptr, ws);
      f->wo += ws;
      size  -= ws;
      buf   += ws;
      if (!size) return osz;
    }
    /* allocate new chunk */
    nc = xcalloc(1, sizeof(*nc));
    if (!f->fp) {
      f->wp = nc;
      f->fp = nc;
      f->rp = nc;
    } else {
      f->wp->next = nc;
      f->wp = nc;
    }
    f->wo = 0;
  }
}

  unsigned long int
memfile_read(MEMFILE *f, void *buf, unsigned long int size)
{
  return 0;
}

  unsigned long int
memfile_dump(MEMFILE *f, FILE *o)
{
  struct chunk_s *p = 0;
  unsigned long   n = 0;

  if (!f || !o) return 0;
  for (p = f->fp; p; p = p->next) {
    n+= fwrite(p->buf, 1, (p == f->wp)?f->wo:CLUSTER, o);
  }
  return n;
}

/*
 * Local variables:
 *  compile-command: "make -C .."
 * End:
 */
