/* $Id: snprintf.c,v 6.7 2002/03/16 10:08:28 cher Exp $ */

/* Copyright (C) 1999 Alexander Chernov */
/* Copyright (C) 1996 DJ Delorie, see COPYING.DJ for details */
/* Copyright (C) 1994 DJ Delorie, see COPYING.DJ for details */

static char const cvsid[]=
"$Id: snprintf.c,v 6.7 2002/03/16 10:08:28 cher Exp $";

#include "lconfig.h"
#include <p_config.h>

#include CONF_DEPFILE("reuse_version.h")

#include "reuse/pprintf.h"
#include "reuse/xalloc.h"
#include "reuse/osdeps.h"
#include "reuse/logger.h"

#include <sys/types.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <locale.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

static char decimal = '.';

/* 11-bit exponent (VAX G floating point) is 308 decimal digits */
#define	MAXEXP		308
#define MAXEXPLD        4952 /* this includes subnormal numbers */
/* 128 bit fraction takes up 39 decimal digits; max reasonable precision */
#define	MAXFRACT	39

#define	BUF		(MAXEXPLD+MAXFRACT+1)	/* + decimal point */

#define	PUTC(ch)	outstr_putc(ch, fp)

struct outstr_t
{
  char   *buf;
  int     size;
  char   *ptr;
  int     left;
};

#if defined __GNUC__
#define M_INLINE inline
#elif defined _MSC_VER
#define M_INLINE __inline
#else
#define M_INLINE
#endif

  static M_INLINE void
outstr_putc(int ch, struct outstr_t *p)
{
  if (p->left > 0) {
    *p->ptr++ = (char) ch;
  }
  p->left--;
}

#if R_HAS_INT64 - 0 == 1
#define ARG(basetype) _ulonglong = \
		flags&LONGDBL ? va_arg(argp, rllong_t basetype) : \
		flags&LONGINT ? va_arg(argp, long basetype) : \
		flags&SHORTINT ? (short basetype)va_arg(argp, int) : \
		va_arg(argp, int)
#else
#define ARG(basetype) _ulonglong = \
		flags&LONGINT ? va_arg(argp, long basetype) : \
		flags&SHORTINT ? (short basetype)va_arg(argp, int) : \
		va_arg(argp, int)
#endif /* R_HAS_INT64 */

#if R_HAS_INT64 - 0 == 1
typedef rullong_t umaxint_t;
typedef rllong_t imaxint_t;
#else
typedef unsigned long umaxint_t;
typedef long imaxint_t;
#endif /* R_HAS_INT64 */

static M_INLINE int todigit(char c)
{
  if (c<='0') return 0;
  if (c>='9') return 9;
  return c-'0';
}
static M_INLINE char tochar(int n)
{
  if (n>=9) return '9';
  if (n<=0) return '0';
  return n+'0';
}

/* have to deal with the negative buffer count kludge */

#define	LONGINT		0x01		/* long integer */
#define	LONGDBL		0x02		/* long double */
#define	SHORTINT	0x04		/* short integer */
#define	ALT		0x08		/* alternate form */
#define	LADJUST		0x10		/* left adjustment */
#define	ZEROPAD		0x20		/* zero (as opposed to blank) pad */
#define	HEXPREFIX	0x40		/* add 0x or 0X prefix */

static char NULL_REP[] = "(null)";

static int
reuse_doprnt(const char *fmt0, va_list argp, struct outstr_t *fp)
{
  const char *fmt;		/* format string */
  int ch;			/* character from fmt */
  int cnt;			/* return value accumulator */
  int n;			/* random handy integer */
  char *t;			/* buffer pointer */
  long double _ldouble;		/* double and long double precision arguments
				   %L.[eEfgG] */
  umaxint_t _ulonglong;         /* integer arguments %[diouxX] */
  int base;			/* base for [diouxX] conversion */
  int dprec;			/* decimal precision in [diouxX] */
  int fieldsz;			/* field size expanded by sign, etc */
  int flags;			/* flags as above */
  int fpprec;			/* `extra' floating precision in [eEfgG] */
  int prec;			/* precision from format (%.3d), or -1 */
  int realsz;			/* field size expanded by decimal precision */
  int size;			/* size of converted field or string */
  int width;			/* width from format (%8d), or 0 */
  char sign;			/* sign prefix (' ', '+', '-', or \0) */
  const char *digs;		/* digits for [diouxX] conversion */
  char buf[BUF];		/* space for %c, %[diouxX], %[eEfgG] */

  char const *fmtstart;
  va_list     fmtargp;

  decimal = localeconv()->decimal_point[0];

  fmt = fmt0;
  digs = "0123456789abcdef";
  for (cnt = 0;; ++fmt)
  {
    while ((ch = *fmt) && ch != '%')
    {
      PUTC (ch);
      fmt++;
      cnt++;
    }
    if (!ch)
      return cnt;
    flags = 0; dprec = 0; fpprec = 0; width = 0;
    fmtstart = fmt; fmtargp = argp;
    prec = -1;
    sign = '\0';
  rflag:
    switch (*++fmt)
    {
    case ' ':
      /*
       * ``If the space and + flags both appear, the space
       * flag will be ignored.''
       *	-- ANSI X3J11
       */
      if (!sign)
	sign = ' ';
      goto rflag;
    case '#':
      flags |= ALT;
      goto rflag;
    case '*':
      /*
       * ``A negative field width argument is taken as a
       * - flag followed by a  positive field width.''
       *	-- ANSI X3J11
       * They don't exclude field widths read from args.
       */
      if ((width = va_arg(argp, int)) >= 0)
	goto rflag;
      width = -width;
      /* FALLTHROUGH */
    case '-':
      flags |= LADJUST;
      goto rflag;
    case '+':
      sign = '+';
      goto rflag;
    case '.':
      if (*++fmt == '*')
	n = va_arg(argp, int);
      else
      {
	n = 0;
	while (isascii(*fmt) && isdigit(*fmt))
	  n = 10 * n + todigit(*fmt++);
	--fmt;
      }
      prec = n < 0 ? -1 : n;
      goto rflag;
    case '0':
      /*
       * ``Note that 0 is taken as a flag, not as the
       * beginning of a field width.''
       *	-- ANSI X3J11
       */
      flags |= ZEROPAD;
      goto rflag;
    case '1': case '2': case '3': case '4':
    case '5': case '6': case '7': case '8': case '9':
      n = 0;
      do {
	n = 10 * n + todigit(*fmt);
      } while (isascii(*++fmt) && isdigit(*fmt));
      width = n;
      --fmt;
      goto rflag;
    case 'L':
      flags |= LONGDBL;
      goto rflag;
    case 'h':
      flags |= SHORTINT;
      goto rflag;
    case 'l':
      if (flags&LONGINT)
	flags |= LONGDBL; /* for 'll' - long long */
      else
	flags |= LONGINT;
      goto rflag;
    case 'c':
      *(t = buf) = va_arg(argp, int);
      size = 1;
      sign = '\0';
      goto pforw;
    case 'D':
      flags |= LONGINT;
      /*FALLTHROUGH*/
    case 'd':
    case 'i':
      ARG(int);
      if ((imaxint_t)_ulonglong < 0)
      {
        _ulonglong = -_ulonglong;
	sign = '-';
      }
      base = 10;
      goto number;
    case 'e':
    case 'E':
    case 'f':
    case 'g':
    case 'G':
      /* fmtstart contains start of the format string */
      {
        char        cvt_format_buf[32];
        char        cvt_out_buf[256];
        char       *pcvt = cvt_format_buf;
        char       *pout = cvt_out_buf;
        char       *hcvt = 0;
        char       *hout = 0;
        int         len  = fmt - fmtstart + 1;
        int         olen = abs(width) + abs(prec) + 32;
        int         cvtn = 0;

        if (len + 1 > 32) {
          hcvt = (char*) xcalloc(1, len + 1);
          pcvt = hcvt;
        }
        memcpy(pcvt, fmtstart, len);
        pcvt[len] = 0;

        if (olen > 256) {
          hout = (char *) xcalloc(1, olen);
          pout = hout;
        }

#if 0
        fprintf(stderr, ">>%s<<\n", pcvt);
#endif /* 0 */

        cvtn = vsprintf(pout, pcvt, fmtargp);
        if (cvtn >= olen) {
          SWERR(("_doprnt: internal buffer overrun"));
        }

        for (; *pout; pout++) {
          PUTC(*pout);
        }

        if (hout) xfree(hout);
        if (hcvt) xfree(hcvt);
        
        if (flags & LONGDBL) {
          _ldouble = va_arg(argp, long double);
        } else {
          _ldouble = (long double)va_arg(argp, double);
        }
      }
      break;
    case 'n':
      if (flags & LONGDBL) {
#if R_HAS_INT64 - 0 == 1
        *va_arg(argp, rllong_t *) = cnt;
#else
	SWERR(("This compiler does not support long long types"));
#endif /* R_HAS_INT64 */
      } else if (flags & LONGINT)
	*va_arg(argp, long *) = cnt;
      else if (flags & SHORTINT)
	*va_arg(argp, short *) = cnt;
      else
	*va_arg(argp, int *) = cnt;
      break;
    case 'O':
      flags |= LONGINT;
      /*FALLTHROUGH*/
    case 'o':
      ARG(unsigned);
      base = 8;
      goto nosign;
    case 'p':
      /*
       * ``The argument shall be a pointer to void.  The
       * value of the pointer is converted to a sequence
       * of printable characters, in an implementation-
       * defined manner.''
       *	-- ANSI X3J11
       */
      /* NOSTRICT */
      _ulonglong = (unsigned long)va_arg(argp, void *);
      base = 16;
      goto nosign;
    case 's':
      if (!(t = va_arg(argp, char *)))
	t = NULL_REP;
      if (prec >= 0)
      {
	/*
	 * can't use strlen; can only look for the
	 * NUL in the first `prec' characters, and
	 * strlen() will go further.
	 */
	char *p			/*, *memchr() */;

	if ((p = memchr(t, 0, prec)))
	{
	  size = p - t;
	  if (size > prec)
	    size = prec;
	}
	else
	  size = prec;
      }
      else
	size = strlen(t);
      sign = '\0';
      goto pforw;
    case 'U':
      flags |= LONGINT;
      /*FALLTHROUGH*/
    case 'u':
      ARG(unsigned);
      base = 10;
      goto nosign;
    case 'X':
      digs = "0123456789ABCDEF";
      /* FALLTHROUGH */
    case 'x':
      ARG(unsigned);
      base = 16;
      /* leading 0x/X only if non-zero */
      if (flags & ALT && _ulonglong != 0)
	flags |= HEXPREFIX;

    nosign:
      /* unsigned conversions */
      sign = '\0';
    number:
      /*
       * ``... diouXx conversions ... if a precision is
       * specified, the 0 flag will be ignored.''
       *	-- ANSI X3J11
       */
      if ((dprec = prec) >= 0)
	flags &= ~ZEROPAD;

      /*
       * ``The result of converting a zero value with an
       * explicit precision of zero is no characters.''
       *	-- ANSI X3J11
       */
      t = buf + BUF;

      if (_ulonglong != 0 || prec != 0)
      {
        /* conversion is done separately since operations
	  with long long are much slower */
#define CONVERT(type) \
	{ \
	  register type _n = (type)_ulonglong; \
	  do { \
	    *--t = digs[_n % base]; \
	    _n /= base; \
	  } while (_n); \
	}
	if (flags&LONGDBL) {
#if R_HAS_INT64 - 0 == 1
	  CONVERT(rullong_t) /* no ; */
#else
	  SWERR(("This compiler does not support long long types"));
#endif /* R_HAS_INT64 */
	} else
	  CONVERT(unsigned long) /* no ; */
#undef CONVERT
        if (flags & ALT && base == 8 && *t != '0')
          *--t = '0';		/* octal leading 0 */
      }

      digs = "0123456789abcdef";
      size = buf + BUF - t;

    pforw:
      /*
       * All reasonable formats wind up here.  At this point,
       * `t' points to a string which (if not flags&LADJUST)
       * should be padded out to `width' places.  If
       * flags&ZEROPAD, it should first be prefixed by any
       * sign or other prefix; otherwise, it should be blank
       * padded before the prefix is emitted.  After any
       * left-hand padding and prefixing, emit zeroes
       * required by a decimal [diouxX] precision, then print
       * the string proper, then emit zeroes required by any
       * leftover floating precision; finally, if LADJUST,
       * pad with blanks.
       */

      /*
       * compute actual size, so we know how much to pad
       * fieldsz excludes decimal prec; realsz includes it
       */
      fieldsz = size + fpprec;
      realsz = dprec > fieldsz ? dprec : fieldsz;
      if (sign)
	realsz++;
      if (flags & HEXPREFIX)
	realsz += 2;

      /* right-adjusting blank padding */
      if ((flags & (LADJUST|ZEROPAD)) == 0 && width)
	for (n = realsz; n < width; n++)
	  PUTC(' ');
      /* prefix */
      if (sign)
	PUTC(sign);
      if (flags & HEXPREFIX)
      {
	PUTC('0');
	PUTC((char)*fmt);
      }
      /* right-adjusting zero padding */
      if ((flags & (LADJUST|ZEROPAD)) == ZEROPAD)
	for (n = realsz; n < width; n++)
	  PUTC('0');
      /* leading zeroes from decimal precision */
      for (n = fieldsz; n < dprec; n++)
	PUTC('0');

      /* the string or number proper */
      for (n = size; n > 0; n--)
        PUTC(*t++);
      /* trailing f.p. zeroes */
      while (--fpprec >= 0)
	PUTC('0');
      /* left-adjusting padding (always blank) */
      if (flags & LADJUST)
	for (n = realsz; n < width; n++)
	  PUTC(' ');
      /* finally, adjust cnt */
      cnt += width > realsz ? width : realsz;
      break;
    case '\0':			/* "%?" prints ?, unless ? is NULL */
      return cnt;
    default:
      PUTC((char)*fmt);
      cnt++;
    }
  }
  /* NOTREACHED */
}

  int
reuse_snprintf(char *str, int size, const char *fmt, ...)
{
  struct outstr_t buf;
  va_list         args;

  buf.buf  = str;
  buf.size = size;
  buf.ptr  = str;
  buf.left = size - 1;

  if (!str)      return 0;
  if (size <= 0) return 0;

  va_start(args, fmt);
  reuse_doprnt(fmt, args, &buf);
  *buf.ptr = 0;
  return buf.ptr - buf.buf;
}

  int
reuse_vsnprintf(char *str, size_t size, const char *fmt, va_list ap)
{
  struct outstr_t buf;

  buf.buf  = str;
  buf.size = size;
  buf.ptr  = str;
  buf.left = size - 1;

  if (!str)      return 0;
  if (size <= 0) return 0;

  reuse_doprnt(fmt, ap, &buf);
  *buf.ptr = 0;
  return buf.ptr - buf.buf;
}

  int
reuse_format_len(char const *fmt, ...)
{
  struct outstr_t buf;
  va_list         args;

  buf.buf = buf.ptr = 0;
  buf.size = buf.left = 0;

  va_start(args, fmt);
  reuse_doprnt(fmt, args, &buf);
  return -buf.left;
}

  int
reuse_vformat_len(char const *fmt, va_list args)
{
  struct outstr_t buf;

  buf.buf = buf.ptr = 0;
  buf.size = buf.left = 0;

  reuse_doprnt(fmt, args, &buf);
  return -buf.left;
}

/* return the formatted string allocated on the heap */

  char *
vssprintf(char const *format, va_list args)
{
  int   len = reuse_vformat_len(format, args);
  char *s   = (char*) xcalloc(1, len + 1);
  if (vsprintf(s, format, args) >= len) {
    SWERR(("vssprintf: reuse_vformat_len returned %d (too few)", len));
  }
  return s;
}

  char *
ssprintf(char const *format, ...)
{
  va_list  args;
  int      len;
  char    *s;

  va_start(args, format);
  len = reuse_vformat_len(format, args);
  s = (char*) xcalloc(1, len + 1);
  if (vsprintf(s, format, args) >= len) {
    SWERR(("vssprintf: reuse_vformat_len returned %d (too few)", len));
  }
  va_end(args);
  return s;
}

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "va_list")
 * End:
 */
