/* -*- mode:c -*- */
/* $Id: os_vfscanf.c,v 1.1 2002/06/08 08:26:41 cher Exp $ */

/* Copyright (C) 2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include "lconfig.h"
#include <p_config.h>
#include <p_integral.h>

#include "reuse/format_io_int.h"

#include <stdio.h>
#include <stdarg.h>

#if CONF_USE_SYSTEM_SCANF - 0 != 1
static int
fscanf_next_char(void *vp)
{
  return getc((FILE *) vp);
}
static int
fscanf_prev_char(void *vp, int c)
{
  return ungetc(c, (FILE *) vp);
}
static struct reuse_scanf_funcs fscanf_vtab =
{
  fscanf_next_char,
  fscanf_prev_char
};
#else
int vfscanf();
#endif /* system vfscanf not to be used */

int
os_vfscanf(FILE *stream, char const *format, va_list args)
{
#if CONF_USE_SYSTEM_SCANF - 0 == 1
  return vfscanf(stream, format, args);
#else
  return _reuse_vscanf(&fscanf_vtab, stream, format, args);
#endif /* vscanf is useable */
}

int
os_fscanf(FILE *stream, char const *format, ...)
{
  va_list args;
  int ret_value;

  va_start(args, format);
#if CONF_USE_SYSTEM_SCANF - 0 == 1
  ret_value = vfscanf(stream, format, args);
#else
  ret_value = os_vfscanf(stream, format, args);
#endif /* scanf is useable */
  va_end(args);
  return ret_value;
}

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make -C .."
 * End:
 */
