/* -*- mode:c -*- */
/* $Id: os_vprintf.c,v 1.1 2002/06/08 08:27:50 cher Exp $ */

/* Copyright (C) 2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include "lconfig.h"
#include <p_config.h>
#include <p_integral.h>

#include "reuse/format_io_int.h"

#include <stdio.h>
#include <stdarg.h>

#if CONF_USE_SYSTEM_PRINTF - 0 != 1
static int
printf_put_char(void *vp, int c)
{
  return putchar(c);
}
static int
printf_put_string(void *vp, char const *str)
{
  return fputs(str, stdout);
}
static int
printf_put_mem(void *vp, char const *str, size_t size)
{
  int r = fwrite(str, 1, size, stdout);
  if (r != size) r = EOF;
  return r;
}
struct reuse_printf_funcs printf_funcs =
{
  printf_put_char,
  printf_put_string,
  printf_put_mem
};
#endif

int
os_vprintf(char const *format, va_list args)
{
#if CONF_USE_SYSTEM_PRINTF - 0 == 1
  return vprintf(format, args);
#else
  return _reuse_vprintf(&printf_funcs, 0, format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

int
os_printf(char const *format, ...)
{
  va_list args;

  va_start(args, format);
#if CONF_USE_SYSTEM_PRINTF - 0 == 1
  return vprintf(format, args);
#else
  return os_vprintf(format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make -C .."
 * End:
 */
