/* -*- mode:c -*- */
/* $Id: os_vscanf.c,v 1.1 2002/06/08 08:26:45 cher Exp $ */

/* Copyright (C) 2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include "lconfig.h"
#include <p_config.h>
#include <p_integral.h>

#include "reuse/format_io_int.h"

#include <stdio.h>
#include <stdarg.h>

#if CONF_USE_SYSTEM_SCANF - 0 != 1
static int
scanf_next_char(void *vp)
{
  return getchar();
}
static int
scanf_prev_char(void *vp, int c)
{
  return ungetc(c, stdin);
}
static struct reuse_scanf_funcs scanf_vtab =
{
  scanf_next_char,
  scanf_prev_char
};
#else
/* it's better not to have a prototype, than to have a wrong one */
int vscanf();
#endif /* local scanf will not be used */

int
os_vscanf(char const *format, va_list args)
{
#if CONF_USE_SYSTEM_SCANF - 0 == 1
  return vscanf(format, args);
#else
  return _reuse_vscanf(&scanf_vtab, 0, format, args);
#endif /* vscanf is useable */
}

int
os_scanf(char const *format, ...)
{
  va_list args;
  int ret_value;

  va_start(args, format);
#if CONF_USE_SYSTEM_SCANF - 0 == 1
  ret_value = vscanf(format, args);
#else
  ret_value = os_vscanf(format, args);
#endif /* scanf is useable */
  va_end(args);
  return ret_value;
}

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make -C .."
 * End:
 */
