/* -*- mode: C -*- */
/* $Id: strtof.c,v 1.2 2002/06/09 07:02:27 cher Exp $ */

/* Copyright (C) 2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include "lconfig.h"
#include <p_config.h>
#include <p_integral.h>

#include "reuse/number_io.h"
#include "reuse/fp_props.h"
#include "reuse/str_utils.h"

#include <errno.h>
#include <ctype.h>
#include <string.h>

#if CONF_HAS_STRTOF - 0 == 1
float strtof();
#endif /* strtof is present */

float
reuse_strtof(char const *str, char **endptr)
{
#if CONF_HAS_STRTOF - 0 == 1
# if CONF_STRTOD_ACCEPTS_HEX - 0 == 1
  return strtof(str, endptr);
# else
  float val;

  if (str && str[0] == '0' && (str[1] == 'x' || str[1] == 'X')) {
    if (readhf(str, endptr, &val) == 1) {
      errno = ERANGE;
    }
    return val;
  }
  return strtof(str, endptr);
# endif /* hexadecimal literals are allowed */
#else
# if CONF_STRTOD_ACCEPTS_HEX - 0 == 1
  float val;

  val = strtod(str, endptr);
  if (reuse_is_infinity_f(&val)) {
    errno = ERANGE;
  }
  return val;
# else
  float val;

  if (str && str[0] == '0' && (str[1] == 'x' || str[1] == 'X')) {
    if (reuse_readhf(str, endptr, &val) == 1) {
      errno = ERANGE;
    }
    return val;
  }
  val = strtod(str, endptr);
  if (reuse_is_infinity_f(&val)) {
    errno = ERANGE;
  }
  return val;
# endif
#endif
}

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE")
 * End:
 */
