/* -*- mode: C -*- */
/* $Id: strtoll.c,v 1.6 2002/06/15 22:17:24 cher Exp $ */

/* Copyright (C) 2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include "lconfig.h"
#include <p_config.h>
#include <p_integral.h>
#include CONF_DEPFILE("reuse_version.h")

#define __REUSE_FORCE_STRTOLL
#include "reuse/number_io.h"

#include <limits.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>

extern const signed char _reuse_letter_to_digit_table[];

rllong_t
reuse_strtoll(char const *str, char **endptr, int base)
{
#if defined CONF_HAS_STRTOLL && CONF_HAS_STRTOLL - 0 == 1
  return strtoll(str, endptr, base);
#else
  unsigned char const *s = (unsigned char const*) str;
  rllong_t val = 0;
  rllong_t range_val;
  int sign = 1;
  int d;

  if (!s) {
    if (endptr) *endptr = (char*) str;
    return R_I64(0);
  }
  while (isspace(*s)) s++;
  if (*s == '-') {
    sign = -1;
    s++;
  } else if (*s == '+') {
    s++;
  }
  if (base == 0) {
    if (*s == '0' && (s[1] == 'X' || s[1] == 'x')) {
      d = _reuse_letter_to_digit_table[s[2]];
      if (d != -1 && d < 16) base = 16;
      else base = 10;
    } else if (*s == '0') {
      base = 8;
    } else if (*s >= '1' && *s <= '9') {
      base = 10;
    }
  }
  if (base < 2 || base > 36) {
    if (endptr) *endptr = (char*) str;
    errno = EINVAL;
    return R_I64(0);
  }
  if (base == 16) {
    if (*s == '0' && (s[1] == 'X' || s[1] == 'x')) {
      d = _reuse_letter_to_digit_table[s[2]];
      if (d != -1 && d < 16) s += 2;
    }
  }
  d = _reuse_letter_to_digit_table[*s];
  if (d == -1 || d >= base) {
    if (endptr) *endptr = (char*) str;
    return R_I64(0);
  }
  if (sign > 0) {
    // read a positive number
    range_val = R_LONG_LONG_MAX;
    val = d;
    s++;
    while (1) {
      d = _reuse_letter_to_digit_table[*s];
      if (d == -1 || d >= base) break;
      if ((val *= base) < 0) goto _range_error;
      if ((val += d) < 0) goto _range_error;
      s++;
    }
  } else {
    // read a negative number
    range_val = R_LONG_LONG_MIN;
    val = -d;
    s++;
    while (1) {
      d = _reuse_letter_to_digit_table[*s];
      if (d == -1 || d >= base) break;
      if ((val *= base) > 0) goto _range_error;
      if ((val -= d) > 0) goto _range_error;
      s++;
    }
  }
  if (endptr) *endptr = (char*) s;
  return val;

 _range_error:
  while (1) {
    d = _reuse_letter_to_digit_table[*s];
    if (d == -1 || d >= base) break;
    s++;
  }
  errno = ERANGE;
  if (endptr) *endptr = (char *) s;
  return range_val;
#endif
}

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE")
 * End:
 */
