#include <limits.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>

#define __REUSE_FORCE_STRTOLL
#include "reuse/osdeps.h"

int errors = 0;
int total = 0;

void
test_ull(char const *str, int base)
{
  char *eptr1, *eptr2;
  unsigned long long val1;
  unsigned long long val2;
  int e1, e2;
  char *okstr;
  int   ok;

  errno = 0;
  eptr1 = (char*) str;
  val1 = strtoull(str, &eptr1, base);
  e1 = errno;
  errno = 0;
  eptr2 = (char*) str;
  val2 = reuse_strtoull(str, &eptr2, base);
  e2 = errno;
  ok = 0;
  if (val1 == val2 && eptr1 == eptr2 && e1 == e2) {
    ok = 1;
    okstr = "OK";
  } else {
    okstr = "NOT OK";
    errors++;
  }
  total++;
  if (!ok) {
    printf("%s : <%s>,%d -> %Lu, %u, %d; %Lu, %u, %d\n",
           okstr, str, base,
           val1, eptr1 - str, e1,
           val2, eptr2 - str, e2);
  }
}

void
test_ll(char const *str, int base)
{
  char *eptr1, *eptr2;
  long long val1;
  long long val2;
  int e1, e2;
  char *okstr;
  int   ok;

  errno = 0;
  val1 = strtoll(str, &eptr1, base);
  e1 = errno;
  errno = 0;
  val2 = reuse_strtoll(str, &eptr2, base);
  e2 = errno;
  ok = 0;
  if (val1 == val2 && eptr1 == eptr2 && e1 == e2) {
    okstr = "OK";
    ok = 1;
  } else {
    okstr = "NOT OK";
    errors++;
  }
  total++;
  if (!ok) {
    printf("%s : <%s>,%d -> %Ld, %u, %d; %Ld, %u, %d\n",
           okstr, str, base,
           val1, eptr1 - str, e1,
           val2, eptr2 - str, e2);
  }
}

char const * const test_strings_16[]=
{
  "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
  "a", "A", "b", "B", "c", "C", "d", "D", "e", "E", "f", "F",
  "10", "0xdddd", "0xffffffff", "ffffffff", "ffffffffffffffff",
  "7fffffffffffffff", "fffffffffffffff", "ffffffffffffff",
  "fffffffffffff", "ffffffffffff", "ffffffffffffffffffff",
  "8000000000000000", "8000000000000001", "+", "-", "   ",
  "0x", "zz", "    u", "     +y", "0xu", "fg",

  0
};

char const * const test_strings_0[]=
{
  "1", "0", "2", "0xdff2", "0777", "078",
  "0xa", "0xx", "1239999999999999999999999999992342",
  "-1233393", "-0383", "-0x3032", "-077777777777777777777777777777777777777",
  0
};

char const * const test_strings_37[]=
{
  "1", 0
};

struct test_seq
{
  int          sflag;
  int          base;
  char const *const *tests;
};

struct test_seq seq[] =
{
  { 0, 16, test_strings_16 },
  { 1, 16, test_strings_16 },

  { 0, 0,  test_strings_0 },
  { 1, 0,  test_strings_0 },

  { 0, 37, test_strings_37 },
  { 1, 37, test_strings_37 },

  { 0, 0 },
};

int main()
{
  int i, j;

  for (i = 0; seq[i].tests; i++) {
    for (j = 0; seq[i].tests[j]; j++) {
      if (!seq[i].sflag) {
        test_ull(seq[i].tests[j], seq[i].base);
      } else {
        test_ll(seq[i].tests[j], seq[i].base);
      }
    }
  }

  printf("%d total tests, %d passed, %d errors\n",
         total, total - errors, errors);

  return (errors>0);
}

/**
 * Local variables:
 *  compile-command: "gcc test3.c -o test3 -Wall -I ../reuse -L ../o -lreuse"
 * End:
 */
