/* $Id: test4.c,v 1.2 2002/01/24 08:56:55 cher Exp $ */

/* Copyright (C) 2002 Alexander Chernov <cher@ispras.ru> */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <reuse/number_io.h>

#include <stdio.h>
#include <math.h>

void out_float(float v, int forced_len)
{
  char buf[128];
  int  len;

  if (forced_len < 0) forced_len = sizeof(buf);
  memset(buf, 0xFF, sizeof(buf));
  len = reuse_writehf(buf, forced_len, &v);
  if (forced_len > 0) {
    printf("%g,%d,%d:>>%s<<%a>>\n", v, len, forced_len, buf, v);
  } else {
    printf("%g,%d,%d:>>%.6s<<\n", v, len, forced_len, buf);
  }
}

void out_double(double v, int forced_len)
{
  char buf[128];
  int  len;

  if (forced_len < 0) forced_len = sizeof(buf);
  memset(buf, 0xFF, sizeof(buf));
  len = reuse_writehd(buf, forced_len, &v);
  if (forced_len > 0) {
    printf("%g,%d,%d:>>%s<<%a>>\n", v, len, forced_len, buf, v);
  } else {
    printf("%g,%d,%d:>>%.6s<<\n", v, len, forced_len, buf);
  }
}

void out_long_double(long double v, int forced_len)
{
  char buf[128];
  int  len;

  if (forced_len < 0) forced_len = sizeof(buf);
  memset(buf, 0xFF, sizeof(buf));
  len = reuse_writehld(buf, forced_len, &v);
  if (forced_len > 0) {
    printf("%Lg,%d,%d:>>%s<<%La>>\n", v, len, forced_len, buf, v);
  } else {
    printf("%Lg,%d,%d:>>%.6s<<\n", v, len, forced_len, buf);
  }
}

int main(void)
{
  printf("=== Testing float output ===\n");

  out_float(0.0f, -1);
  out_float(1.0f, -1);
  out_float(2.0f, -1);
  out_float(0.1f, -1);
  out_float(1.112e23f, -1);
  out_float(-1.0f, -1);
  out_float(-2.0f, -1);
  out_float(-0.1f, -1);
  out_float(-1.112e23f, -1);
  out_float(0.0458333, -1);
  out_float((float) 1e300, -1);
  out_float((float) -1e300, -1);
  out_float((float) -1e-100, -1);
  out_float(0.0f/0.0f, -1);

  printf("String truncating testing\n");

  out_float(0.0f, 2);
  out_float(0.0f, 0);
  out_float(1.112e23f, 4);
  out_float(1.112e23f, 1);
  out_float(-1.112e23f, 2);
  out_float(-1.112e23f, 0);

  printf("=== Testing double output ===\n");
  out_double(0.0, -1);
  out_double(1.0, -1);
  out_double(2.0, -1);
  out_double(0.1, -1);
  out_double(1.112e23, -1);
  out_double(-1.0, -1);
  out_double(-2.0, -1);
  out_double(-0.1, -1);
  out_double(-1.112e23, -1);
  out_double((double) 1e1300L, -1);
  out_double((double) -1e1300L, -1);
  out_double((double) -1e-1100L, -1);
  out_double(0.0/0.0, -1);

  printf("=== Testing long double output ===\n");
  out_long_double(0.0L, -1);
  out_long_double(1.0L, -1);
  out_long_double(2.0L, -1);
  out_long_double(0.1L, -1);
  out_long_double(1.112e23L, -1);
  out_long_double(-1.0L, -1);
  out_long_double(-2.0L, -1);
  out_long_double(-0.1L, -1);
  out_long_double(-1.112e23L, -1);
  out_long_double(1e22300L, -1);
  out_long_double(-1e22300L, -1);
  out_long_double(-1e-22100L, -1);
  out_long_double(0.0L/0.0L, -1);

  return 0;
}

/*
 * Local variables:
 *  compile-command: "gcc -O2 -s -I.. -I~/c-sema/include/ix86-linux test4.c -L../.obj-ix86-linux -Wall -o test4 -lreuse -lm"
 * End:
 */
