#!/bin/sh
# $Id: runmake.sh,v 6.4 2001/06/23 07:55:36 cher Exp $

# Copyright (C) 1998-2001 Alexander Chernov <cher@ispras.ru>

#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

prefix_file=
[ -x ./etc/env_prefix.sh ] && prefix_file=./etc/env_prefix.sh
[ -x ./env_prefix.sh ]     && prefix_file=./env_prefix.sh

if [ x"${PROJECT_ENV_PREFIX}" = x ]
then
    if [ x"${prefix_file}" = x ]
    then
        echo "Cannot guess project environment prefix" >&2
        exit 1
    fi
    . ${prefix_file}
    if [ x"${PROJECT_ENV_PREFIX}" = x ]
    then
        echo "Cannot guess project environment prefix" >&2
        exit 1
    fi
fi

prj_home_name=${PROJECT_ENV_PREFIX}_HOME
prj_config_name=${PROJECT_ENV_PREFIX}_CONFIG
prj_mode_name=${PROJECT_ENV_PREFIX}_MODE
prj_query_name=${PROJECT_ENV_PREFIX}_DO_QUERY

eval 'prj_home=$'${prj_home_name}
eval 'prj_config=$'${prj_config_name}
eval 'prj_mode=$'${prj_mode_name}
eval 'prj_query=$'${prj_query_name}

no_conf_detect=0

while :
do
  case x"$1" in
    x-q)
      prj_query=1
      eval ${prj_query_name}=${prj_query}
      export ${prj_query_name}
      shift
      ;;
    x-r)
      no_conf_detect=1
      shift
      ;;
    *)
      break
      ;;
  esac
done

MAKE=${MAKE:-make}

if [ x"$no_conf_detect" = x0 ]
then
  exefile=$0
  exename=`basename $exefile`
  exepath=`dirname $exefile`
  if [ "$exepath" = "." ]
  then
    abspath=`pwd`
    absfile=`pwd`/$exename
  elif [ "$exepath" = / ]
  then
    abspath=/
  else
    abspath=`cd $exepath; pwd`
  fi

  if [ $abspath = '/' ]
  then
    echo "cannot work with root directory" >&2
  exit 1
  fi

  absfile=${abspath}/${exename}

  if [ x"${prj_home}" = x ]
  then
    prj_home=${abspath}
    eval ${prj_home_name}=${prj_home}
    export ${prj_home_name}
  fi

  args=$*
  if [ x"${prj_config}" = x ]
  then
    if prj_config=`./etc/config.guess`
    then
        :
    else
        echo "Cannot guess the canonic configuration name" >&2
        exit 1
    fi
    eval ${prj_config_name}=${prj_config}
    export ${prj_config_name}
  fi
  set -- $args

  echo "${prj_home_name}=${prj_home}"   >&2
  echo "${prj_config_name}=${prj_config}" >&2
  echo "${prj_mode_name}=${prj_mode}"   >&2
fi

if [ $# -lt 2 ]
then
  echo "Synopsis: runmake.sh [ options ] target dirs"
  exit 0
fi

target=$1
shift

flag=1

for i in $*
do
  if [ x"${prj_query}" = x1 -a -t 0 -a -t 1 ]
  then
    printf "Proceed with directory" $i "[Y/N]?"
    if [ $cont = "n" -o $cont = "N" ]; then continue; fi
  fi

  if [ ! -d $i ]
  then
    echo "Directory $i does not exist, skipped"
    continue
  fi

  while true
  do
    (cd $i; ${MAKE} $target)
    retcode=$?
    if [ $retcode != 0 ]
    then
      if [ -t 0 -a -t 1 ]
      then
        echo "Make failed on" $i
        printf "Continue anyway[Y/R/Q]?"
        read cont
        if [ "$cont" = "Y" -o "$cont" = "y" ]; then break
        elif [ "$cont" = "R" -o "$cont" = "r" ]; then continue
        else exit $retcode; fi
      else
        exit $retcode
      fi
    else
      break
    fi
  done
done

exit 0
