#!/bin/sh
# $Id: build.sh,v 6.17 2002/06/15 22:21:35 cher Exp $

# Copyright (C) 1998-2002 Alexander Chernov <cher@ispras.ru>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

rm_dep () {
  [ x"${REALD}" = x ] && return 0
#  [ x"${REALD}" = xdep -a -f ${REALD}/.no_clean ] && return 0
  rm -rf ${REALD}
  return 0
}

rm_gen () {
  [ x"${REALG}" = x ] && return 0
  [ x"${REALG}" = xgen -a -f ${REALG}/.no_clean ] && return 0
  rm -rf ${REALG}
  return 0
}

if [ x"${PROJECT_ENV_PREFIX}" = x ]
then
  if [ -f ../env_prefix.sh ]; then :
  else
    echo "Cannot get project environment prefix" >&2
    exit 1
  fi
  . ../env_prefix.sh
  if [ x"${PROJECT_ENV_PREFIX}" = x ]
  then
    echo "Cannot get project environment prefix" >&2
    exit 1
  fi
fi

home_var_name=${PROJECT_ENV_PREFIX}'_HOME'
config_var_name=${PROJECT_ENV_PREFIX}'_CONFIG'
mode_var_name=${PROJECT_ENV_PREFIX}'_MODE'

eval L_PRJ_HOME='$'${home_var_name}
eval L_PRJ_CONFIG='$'${config_var_name}
eval L_PRJ_MODE='$'${mode_var_name}

export L_PRJ_HOME L_PRJ_CONFIG L_PRJ_MODE

# Try to read the configuration file
if [ x"${L_PRJ_HOME}" = x ]
then
  echo "Environment variable ${home_var_name} is not set" >&2
  exit 1
fi
if [ x"${L_PRJ_CONFIG}" = x ]
then
  echo "Environment variable ${config_var_name} is not set" >&2
  exit 1
fi

ff=${L_PRJ_HOME}/etc/.cfg-${L_PRJ_CONFIG}/compile.sh
[ -f ${ff} ] && . ${ff}

opt_use_deps=1
[ -f ./options.sh ] && . ./options.sh

# Check that this component is disabled
if [ x"${opt_disable_var}" != x ]
then
  eval tempval='${'${opt_disable_var}'}'
  [ x"${tempval}" = x1 ] && exit 0
fi

no_compilation=0
ff=${L_PRJ_HOME}/etc/.cfg-${L_PRJ_CONFIG}/mode-${L_PRJ_MODE}.make
msg="Compilation mode file for mode ${L_PRJ_MODE}"
[ x"${L_PRJ_MODE}" = x ] && msg="Default compilation mode file"
if [ -f "${ff}" ]; then :;
else
  [ x"$@" != xdistclean ] && echo "${msg} does not exist" >&2
  no_compilation=1
fi

if [ ${no_compilation} != 1 ]
then
  eval `grep 'ENABLE_LN' $ff`
  case x"$ENABLE_LN" in
    x0) SLN=;;
    x1) SLN="ln -s";;
    *)  echo "Failed to read ENABLE_LN configuration variable" >&2; exit 1;;
  esac

  REALO=.obj-${L_PRJ_CONFIG}
  [ x"${CONF_NO_ARCH_OBJDIR}" = x1 ] && REALO=obj
  [ x"${opt_use_obj}" = x0 ] && REALO=
  REALG=.gen-${L_PRJ_CONFIG}
  [ x"${CONF_NO_ARCH_GENDIR}" = x1 ] && REALG=gen
  [ x"${opt_use_gen}" = x0 ] && REALG=
  REALD=.dep-${L_PRJ_CONFIG}
  [ x"${CONF_NO_ARCH_DEPDIR}" = x1 ] && REALD=dep
  [ x"${opt_use_deps}" = x0 ] && REALD=
  REALP=.pic-${L_PRJ_CONFIG}
  [ x"${CONF_NO_ARCH_OBJDIR}" = x1 ] && REALP=pic
  [ x"${opt_use_pic}" != x1 -o x"${CONF_DLL}" = x0 ] && REALP=
  GENG=gen
  GEND=dep
  [ x"${opt_save_genfiles}" = x0 ] && GENG=
  [ x"${opt_save_depfiles}" = x0 ] && GEND=
  export REALO REALG REALD REALP GENG GEND
fi # no_compilation

case $1 in
  clean)
    rm -rf core o g p d
    [ x"${REALO}" != x ] && rm -rf ${REALO}
    [ x"${REALP}" != x ] && rm -rf ${REALP}
    exit 0
    ;;
  genclean)
    rm -rf core o g p d
    [ x"${REALO}" != x ] && rm -rf ${REALO}
    [ x"${REALP}" != x ] && rm -rf ${REALP}
    rm_gen
    rm_dep
    exit 0
    ;;
  confclean)
    exit 0
    ;;
  distclean)
    rm_gen
    rm_dep
    [ x"${REALP}" = xpic ] && rm -rf ${REALP}
    [ x"${REALO}" = xobj ] && rm -rf ${REALO}
    rm -rf core .obj-* .pic-* .gen-* .dep-* o g p d TAGS
    exit 0
    ;;
  save_genfiles)
    if [ x"${GENG}" != x -a x"${REALG}" != x ]
    then
      if [ x"${REALG}" != x"${GENG}" ]
      then
        rm -rf ${GENG}
        mv ${REALG} ${GENG}
      fi
      touch ${GENG}/.no_clean
    fi
#    if [ x"${GEND}" != x -a x"${REALD}" != x ]
#    then
#      if [ x"${REALD}" != x"${GEND}" ]
#      then
#        rm -rf ${GEND}
#        mv ${REALD} ${GEND}
#      fi
#    fi
    exit 0
    ;;
  remove_genfiles)
    if [ x"${GENG}" != x ]
    then
      rm -rf ${GENG}
    fi
#    if [ x"${GEND}" != x ]
#    then
#      rm -rf ${GEND}
#    fi
#    exit 0
    ;;
esac

[ $no_compilation != 1 ] || exit 1

[ x"${CONF_NO_GEN_REMAKE}" = x1 ] && REALG=
[ x"${CONF_NO_DEP_REMAKE}" = x1 ] && REALD=
lnk_g=g
[ x"${REALG}" = xgen ] && lnk_g=
lnk_d=d
[ x"${REALD}" = xdep ] && lnk_d=
lnk_p=p
[ x"${REALP}" = xpic ] && lnk_p=
lnk_o=o
[ x"${REALO}" = xobj ] && lnk_o=

stage=0
for qq in "${REALO}" "${lnk_o}" "${REALP}" "${lnk_p}" "${REALG}" "${lnk_g}" "${REALD}" "${lnk_d}"
do
  if [ $stage = 0 ]
  then
    objdir=$qq
    stage=1
    continue
  fi

  lndir=$qq
  stage=0

  [ x"${objdir}" != x ] || continue
  if [ -d ${objdir} ]
  then
    :
  else
    mkdir ${objdir} || exit 1
  fi

  [ x"$SLN" != x ] || continue
  [ x"$lndir" != x ] || continue

  rm -f $lndir
  $SLN $objdir $lndir || exit 1
done

[ -x ./premake.sh ] && ./premake.sh

case $1 in
  all|install|install_only_compile)
    if [ x"${opt_use_deps}" != x0 ]
    then
      if [ x"${CONF_NO_DEP_REMAKE}" != x1 ]
      then
        > ${REALD}/deps.make
        ${MAKE:-make} -f build.make deps || exit 1
      fi
    fi
    ;;
  deps)
    if [ x"${CONF_NO_DEP_REMAKE}" != x1 ]
    then
      [ x"${opt_use_deps}" != x0 ] && > ${REALD}/deps.make
    fi
    ;;
  tags)
    ;;
esac

exec ${MAKE:-make} -f build.make "$@"
