#!/bin/sh
# $Id: mkwrapper.sh,v 6.6 2001/04/17 21:17:10 cher Exp $

# Copyright (C) 1999-2001 Alexander Chernov <cher@ispras.ru>

#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

# Usage:
#   mkwrapper.sh [ options ] output-name
# Options:
# --share             - invoke from share directory
# --libexec           - invoke from libexec directory [default]
# --extra-dir         - extra invokation directory
# --suffix            - invokation suffix
# --extra-args        - extra arguments passed to the tool
# --start-name        - explict name of executable to start
# --simple-home       - simple home detection code

share_option=0
simple_option=0
extra_dir=
suffix=
extra_args=
start_name=

while :
do
    case x"$1" in
        x--share)
            share_option=1
            ;;
        x--libexec)
            share_option=0
            ;;
        x--extra-dir)
            extra_dir=$2
            shift
            ;;
        x--suffix)
            suffix=$2
            shift
            ;;
        x--extra-args)
            extra_args=$2
            shift
            ;;
        x--start-name)
            start_name=$2
            shift
            ;;
        x--simple-home)
            simple_option=1
            ;;
        x-*)
            echo "mkwrapper.sh: unknown option" >&2
            exit 1
            ;;
        x*)
            break
            ;;
    esac
    shift
done

if [ x"$1" = x ]
then
    echo "mkwrapper.sh: output file name must be specified" >&2
    exit 1
fi
out=$1

if [ x"${PROJECT_ENV_PREFIX}" = x ]
then
  if [ -f ../env_prefix.sh ]
  then
  . ../env_prefix.sh
  elif [ -f ../../env_prefix.sh ]
  then
  . ../../env_prefix.sh
  else
    echo "Cannot get project environment prefix" >&2
    exit 1
  fi
  if [ x"${PROJECT_ENV_PREFIX}" = x ]
  then
    echo "Cannot get project environment prefix" >&2
    exit 1
  fi
fi

home_var_name=${PROJECT_ENV_PREFIX}'_HOME'
config_var_name=${PROJECT_ENV_PREFIX}'_CONFIG'
mode_var_name=${PROJECT_ENV_PREFIX}'_MODE'

> ${out}
echo '#!/bin/sh' >> ${out}
echo '# This script is generated by:' >> ${out}
echo '# $Id: mkwrapper.sh,v 6.6 2001/04/17 21:17:10 cher Exp $' >> ${out}
echo '# Generation date:' >> ${out}
echo '# '`date` >> ${out}
echo >> ${out}
cat >> ${out} <<EOF
args=\$@
arg0=\$0
EOF
echo >> ${out}
if [ x"${start_name}" = x ]
then
  echo 'startname=`basename $0`'${suffix} >> ${out}
else
  echo 'startname='${start_name} >> ${out}
fi

if [ ${simple_option} = 1 ]
then
cat >> ${out} <<EOF
if [ x"\${${home_var_name}}" = x ]
then
  ${home_var_name}=\`dirname \$0\`/..
  export ${home_var_name}
fi
EOF
else
cat >> ${out} <<EOF
startdir=\`dirname \$0\`
startdir=\`cd \$startdir; pwd\`
upperdir=\`basename \$startdir\`

repdir=\`dirname \$startdir\`
if [ "\$repdir" = "" -o "\$repdir" = "." ]
then
  echo "\$startname: cannot start from \$startdir" >&2
  exit 0
fi
realrepdir=\`cd \$repdir; pwd\`

if [ x"\${${home_var_name}}" = x ]
then
  ${home_var_name}=\${realrepdir}
  export ${home_var_name}
fi
EOF
fi
cat >> ${out} <<EOF

if [ x"\${${config_var_name}}" = x ]
then
  ${config_var_name}=\`\${${home_var_name}}/etc/config.guess\`
  if [ x"\${${config_var_name}}" = x ]
  then
    echo "\$arg0: Cannot guess configuration name" >&2
    exit 1
  fi
  export ${config_var_name}
fi

EOF

pfx='${'${home_var_name}'}/libexec/${'${config_var_name}'}'${extra_dir}
[ ${share_option} = 1 ] && pfx='${'${home_var_name}'}/share'${extra_dir}

echo exec ${pfx}/'${startname}' ${extra_args} '${1+"$@"}' >> ${out}

chmod +x ${out}

exit 0
