/* $Id: vcw.c,v 6.2 2001/06/23 08:02:49 cher Exp $ */

/* Copyright (C) 2000-2001 Alexander Chernov cher@ispras.ru */

/*
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

/**
 * This simple program provides a gcc-like interface to
 * MS Visual C command line compiler CL
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <windows.h>

static char const rcsid[]=
"$Id: vcw.c,v 6.2 2001/06/23 08:02:49 cher Exp $";

/* memory manipulation routines (reuse/xalloc) */
/* ================ XALLOC.H STARTS HERE ================== */

extern void *xmalloc(size_t size);
extern void *xcalloc(size_t nelem, size_t elsize);
extern void *xrealloc(void *ptr, size_t newsize);
extern void xfree(void *ptr);
extern char *xstrdup(char const*);
extern char *xmemdup(char const *, size_t size);

#if defined __GNUC__
#define XCALLOC(p,s)  ((p) = (typeof(p)) xcalloc((s), sizeof((p)[0])))
#define XREALLOC(p,s) ((p) = (typeof(p)) xrealloc((p), (s) * sizeof((p)[0])))
#else /* __GNUC__ */
#define XCALLOC(p,s)  ((p) = xcalloc((s), sizeof((p)[0])))
#define XREALLOC(p,s) ((p) = xrealloc((p), (s) * sizeof((p)[0])))
#endif /* __GNUC__ */

#define XMEMMOVE(d,s,c) (memmove((d),(s),(c)*sizeof(*(d))))
#define XMEMZERO(d,c)   (memset((d),0,(c)*sizeof(*(d))))
#define XEXPAND2(a)     (xexpand2(&(a),sizeof((a).v[0])))

#define XOFFSET(type,field)       ((long) &((type*) 0)->field)
#define XDEREF(type,base,offset)  (((type*) (((char*) &(base)) + (offset))))
#define XPDEREF(type,base,offset) (((type*) (((char*) (base)) + (offset))))

/* s1 and s2 both dropped after merging */
extern char *xstrmerge0(char *s1, char *s2);
/* only s1 dropped after merging */  
extern char *xstrmerge1(char *s1, char const *s2);
/* neither s1 nor x2 are dropped after merging */
extern char *xstrmerge2(char const *s1, char const *s2);
/* only s2 dropped after merging */
extern char *xstrmerge3(char const *s1, char *s2);

/* extendable array of strings */
typedef struct strarray_t
{
  int    a;
  int    u;
  char **v;
} strarray_t;

/* extendable array of ints */
typedef struct intarray_t
{
  int    a;
  int    u;
  int   *v;
} intarray_t;

/* generic extendable array */
typedef struct genarray_t
{
  int    a;
  int    u;
  void  *v;
} genarray_t;

extern void  xexpand(strarray_t *);
extern void  xexpand2(/* array, elsize */);
extern void  xexpand3(/* array, elsize */);
extern void  xexpand4(/* array, elsize, newsize */);

extern void  xstrarrayfree(strarray_t *);

/* ==================== XALLOC.H ENDS HERE ==================== */

char *strlasterror(void)
{
  static char buf[1024];

  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                NULL,
                GetLastError(),
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                buf, sizeof(buf), NULL);
  return buf;
}

char *os_DirName(char const *str);

struct mtable
{
        char *dospath;
        char *mntpath;
        int   mntlen;
};
struct mtable_arr
{
  int            a, u;
  struct mtable *v;
} mount_table;

void read_mount_table(void)
{
  HKEY     software_rkey = 0;
  HKEY     cygnus_rkey = 0;
  HKEY     cygwin_rkey = 0;
  HKEY     mounts_rkey = 0;
  int      i;
  char     subkey[1024];
  int      subkey_len;
  FILETIME change_time;
  char     native[1024];
  int      native_len;
  HKEY     point_rkey;
  int      value_type;
  
  /* HKEY_CURRENT_USER\SOFTWARE\Cygnus Solutions\Cygwin\mounts v2 */
  if (RegOpenKeyEx(HKEY_CURRENT_USER, "SOFTWARE", 0,
                   KEY_READ, &software_rkey) != ERROR_SUCCESS) {
    goto cleanup;
  }
  if (RegOpenKeyEx(software_rkey, "Cygnus Solutions", 0,
                   KEY_READ, &cygnus_rkey) != ERROR_SUCCESS) {
    goto cleanup;
  }
  if (RegOpenKeyEx(cygnus_rkey, "Cygwin", 0,
                   KEY_READ, &cygwin_rkey) != ERROR_SUCCESS) {
    goto cleanup;
  }
  if (RegOpenKeyEx(cygwin_rkey, "mounts v2", 0,
                   KEY_READ, &mounts_rkey) != ERROR_SUCCESS) {
    goto cleanup;
  }

  i = 0;
  subkey_len = sizeof(subkey);
  while (RegEnumKeyEx(mounts_rkey, i,
                      subkey, (DWORD*) &subkey_len, NULL, NULL, NULL,
                      &change_time) == ERROR_SUCCESS) {
    i++;
    subkey_len = sizeof(subkey);

    if (RegOpenKeyEx(mounts_rkey, subkey, 0, KEY_READ,
                     &point_rkey) != ERROR_SUCCESS) continue;
    native_len = sizeof(native);
    if (RegQueryValueEx(point_rkey, "native", NULL,
                        (DWORD*) &value_type, native,
                        (DWORD*) &native_len) == ERROR_SUCCESS) {
      XEXPAND2(mount_table);
      mount_table.v[mount_table.u].dospath = xstrdup(native);
      mount_table.v[mount_table.u].mntpath = xstrdup(subkey);
      mount_table.v[mount_table.u].mntlen = strlen(subkey);
      mount_table.u++;
    }
    RegCloseKey(point_rkey);
  }

 cleanup:
  if (software_rkey != 0) RegCloseKey(software_rkey);
  if (cygnus_rkey != 0) RegCloseKey(cygnus_rkey);
  if (cygwin_rkey != 0) RegCloseKey(cygwin_rkey);
  if (mounts_rkey != 0) RegCloseKey(mounts_rkey);
}

void check_visual_studio(int *major, char *msvcdir, char *osdir, 
                         char *commondir, char *devdir)
{
  int      value_type;
  int      value_len;
  char     value_str[1024];
  HKEY     devstudio_key = 0;
  FILETIME developer_studio_version_change_time;
  char     developer_studio_version[1024];
  int      developer_studio_version_len;
  int      i;

  int      cur_major, cur_minor, max_major = -1, max_minor = 0;
  char     max_version[1024];

  char     devstudio_regpath[1024];
  char    *env_os = 0;
  char    *ms_dir;

  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                   "Software\\Microsoft\\DevStudio",
                   0, KEY_READ, &devstudio_key) != ERROR_SUCCESS) {
    fprintf(stderr, "vcw: Microsoft Visual Studio is not installed\n");
    exit(1);
  }

  i = 0;
  developer_studio_version_len = sizeof(developer_studio_version);
  while (RegEnumKeyEx(devstudio_key, i,
                      developer_studio_version,
                      (DWORD*) &developer_studio_version_len,
                      NULL, NULL, NULL,
                      &developer_studio_version_change_time)
         == ERROR_SUCCESS) {
    i++;
    developer_studio_version_len = sizeof(developer_studio_version);

    //fprintf(stderr, "Found version %s\n", developer_studio_version);
    sscanf(developer_studio_version, "%d.%d",
           &cur_major, &cur_minor);
    if (cur_major > max_major ||
        (cur_major == max_major && cur_minor > max_minor)) {
      max_major = cur_major;
      max_minor = cur_minor;
      strcpy(max_version, developer_studio_version);
    }
  }
  if (max_major == -1) {
    fprintf(stderr, "vcw: Microsoft Visual Studio is not installed\n");
    exit(1);
  }
  RegCloseKey(devstudio_key);

  //fprintf(stderr, "Chosen version %s\n", max_version);

  sprintf(devstudio_regpath,
          "%s\\%s\\%s",
          "Software\\Microsoft\\DevStudio",
          max_version,
          "Products\\Microsoft Visual C++");

  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                   devstudio_regpath,
                   0, KEY_READ, &devstudio_key) != ERROR_SUCCESS) {
    goto error;
  }

  value_len = sizeof(value_str);
  if (RegQueryValueEx(devstudio_key, "ProductDir", NULL,
                      (DWORD*) &value_type, value_str,
                      (DWORD*) &value_len) != ERROR_SUCCESS) {
    goto error;
  }
  //fprintf(stderr, "MSVCDir=%s\n", value_str);
  strcpy(msvcdir, value_str);

  *major = max_major;
  env_os = getenv("OS");
  if (env_os && !strcmp(env_os, "Windows_NT")) {
    strcpy(osdir, "WINNT");
  } else {
    strcpy(osdir, "WIN95");
  }
  //fprintf(stderr, "VCOSDir=%s\n", osdir);

  ms_dir = os_DirName(value_str);
  if (max_major == 6) {
    sprintf(commondir, "%s\\COMMON", ms_dir);
    sprintf(devdir, "%s\\msdev98", commondir);
  } else {
    fprintf(stderr, "This version of Visual Studio is not supported\n");
    exit(1);
  }

  //fprintf(stderr, "VSCommonDir=%s\n", commondir);
  //fprintf(stderr, "MSDevDir=%s\n", devdir);

  return;

 error:
  fprintf(stderr, "vcw: Microsoft Visual Studio is not installed\n");
  exit(1);
}

int do_system(char *cmd)
{
  STARTUPINFO         sinfo;
  PROCESS_INFORMATION pinfo;
  int                 exitcode;

  //fprintf(stderr, "!%s\n", cmd);

  XMEMZERO(&sinfo, 1);
  XMEMZERO(&pinfo, 1);

  sinfo.cb = sizeof(sinfo);
  sinfo.dwFlags = 0; // STARTF_USESTDHANDLES;
  sinfo.lpReserved = NULL;
  sinfo.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
  sinfo.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
  sinfo.hStdError = GetStdHandle(STD_ERROR_HANDLE);

  //fprintf(stderr, "!!0x%08x,0x%08x,0x%08x\n",
  //        sinfo.hStdInput, sinfo.hStdOutput, sinfo.hStdError);

  if (!CreateProcess(NULL, cmd, NULL, NULL, TRUE,
                     NORMAL_PRIORITY_CLASS, NULL, NULL, &sinfo, &pinfo)) {
    fprintf(stderr, "vcw: CreateProcess failed: %s (%d)\n",
            strlasterror(), (int) GetLastError());
    exit(1);
  }

  WaitForSingleObject(pinfo.hProcess, INFINITE);
  GetExitCodeProcess(pinfo.hProcess, (DWORD*) &exitcode);
  return exitcode;
}

void convert_slashes1(char*);
void convert_slashes2(char*);
void smash_slashes(char*);

void process_mounts(char *path)
{
  int   maxlen = 0;
  int   maxind = -1;
  int   i;
  char *t;

  //printf("u = %d\n", mount_table.u);

  for (i = 0; i < mount_table.u; i++) {
    //printf("search: %s, %d\n",
    //       mount_table.v[i].mntpath,
    //       mount_table.v[i].mntlen);
    if (!strncmp(path, mount_table.v[i].mntpath,
                 mount_table.v[i].mntlen) &&
        mount_table.v[i].mntlen > maxlen) {
      maxind = i;
      maxlen = mount_table.v[i].mntlen;
    }
  }
  //printf("index: %d, %d\n", maxind, maxlen);
  if (maxind < 0) return;

  t = xstrdup(path + maxlen);
  strcpy(path, mount_table.v[maxind].dospath);
  strcat(path, "/");
  strcat(path, t);
  xfree(t);
  convert_slashes1(path);
  smash_slashes(path);
  convert_slashes2(path);
}

void convert_slashes1(char *path)
{
  char *s = path;

  // convert \\ to /
  for (; *s; s++) if (*s == '\\') *s = '/';
}

void convert_slashes2(char *path)
{
  char *s = path;
  
  /* convert / to \\ */
  for (; *s; s++) if (*s == '/') *s = '\\';
}

void smash_slashes(char *path)
{
  char *s;

  if (path[0] == '/' && path[1] == '/' &&
      isalpha(path[2])) {
    if (path[3] == '/') {
      path[0] = path[2];
      path[1] = ':';
      path[2] = '/';
    } else if (path[3] == 0) {
      path[0] = path[2];
      path[1] = ':';
      path[2] = '/';
    }
  }

  for (s = path; *s; s++) {
    if (*s == '/' && s[1] == '/') {
      memmove(s, s + 1, strlen(s));
      s--;
    }
  }
}

void process_cygdrive(char *path)
{
  if (!strncmp(path, "/cygdrive/", 10) && isalpha(path[10]) &&
      path[11] == '/') {
    path[0] = path[10];
    path[1] = ':';
    memmove(path + 2, path + 11, strlen(path + 11) + 1);
  }
}

char *fix_path(char const *old_path)
{
  char *new_path = (char*) xcalloc(2048, 1);
  strcpy(new_path, old_path);
  convert_slashes1(new_path);
  smash_slashes(new_path);
  process_cygdrive(new_path);
  process_mounts(new_path);
  convert_slashes2(new_path);

  return new_path;
}

char *space_merge(char *s1, char *s2) {
  char *s;

  if (!s1 && !s2) return xstrdup("");
  if (!s1) return s2;
  if (!s2) return s1;
  if (!*s1) {
    xfree(s1);
    return s2;
  }
  if (!*s2) {
    xfree(s2);
    return s1;
  }

  s = (char*) xmalloc(strlen(s1) + strlen(s2) + 2);
  strcpy(s, s1);
  strcat(s, " ");
  strcat(s, s2);
  xfree(s1);
  xfree(s2);
  return s;
}

void check_argument(int i, int argc, char *opt)
{
  if (i + 1 >= argc) {
    fprintf(stderr, "vcw: argument required after %s\n", opt);
    exit(1);
  }
}

char *prefix_compile = 0;
char *user_compile = 0;
char *suffix_compile = 0;
char *prefix_link = 0;
char *user_link = 0;
char *suffix_link = 0;
char *cmdline = 0;

char *output_name = 0;
char *libpath = 0;

int number_of_files = 0;
int stop_stage;
int lib_mode;

char MSVCDir[1024];
char env_include[1024];
char env_lib[1024];
char env_path[4096];
char VSCommonDir[1024];
char MSDevDir[1024];
char VCOSDir[1024];

int  VC_major;

int main(int argc, char const *argv[])
{
  int   i;
  char *path = 0;

  check_visual_studio(&VC_major, MSVCDir, VCOSDir, VSCommonDir, MSDevDir);
  read_mount_table();

  // parse command line
  for (i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "-I")) {
      check_argument(i, argc, "-I");
      path = fix_path(argv[++i]);
      user_compile = space_merge(user_compile, xstrdup("/I"));
      user_compile = xstrmerge0(user_compile, path);
    } else if (!strncmp(argv[i], "-I", 2)) {
      path = fix_path(argv[i] + 2);
      user_compile = space_merge(user_compile, xstrdup("/I"));
      user_compile = xstrmerge0(user_compile, path);
    } else if (!strcmp(argv[i], "-D")) {
      check_argument(i, argc, "-D");
      user_compile = space_merge(user_compile, xstrdup("/D"));
      user_compile = xstrmerge1(user_compile, argv[++i]);
    } else if (!strncmp(argv[i], "-D", 2)) {
      user_compile = space_merge(user_compile, xstrdup("/D"));
      user_compile = xstrmerge1(user_compile, argv[i] + 2);
    } else if (!strcmp(argv[i], "-L")) {
      check_argument(i, argc, "-L");
      path = fix_path(argv[++i]);
      user_link = space_merge(user_link, xstrdup("/libpath:"));
      user_link = xstrmerge0(user_link, path);
    } else if (!strncmp(argv[i], "-L", 2)) {
      path = fix_path(argv[i] + 2);
      user_link = space_merge(user_link, xstrdup("/libpath:"));
      user_link = xstrmerge0(user_link, path);
    } else if (!strcmp(argv[i], "-l")) {
      check_argument(i, argc, "-l");
      path = fix_path(argv[++i]);
      user_compile = space_merge(user_compile, path);
      user_compile = xstrmerge0(user_compile, ".LIB");
    } else if (!strncmp(argv[i], "-l", 2)) {
      path = fix_path(argv[i] + 2);      
      user_compile = space_merge(user_compile, path);
      user_compile = xstrmerge0(user_compile, ".LIB");
    } else if (!strcmp(argv[i], "-o")) {
      check_argument(i, argc, "-o");
      output_name = fix_path(argv[++i]);
    } else if (!strncmp(argv[i], "-o", 2)) {
      output_name = fix_path(argv[i] + 2);
    } else if (!strcmp(argv[i], "-mlib")) {
      lib_mode = 1;
    } else if (!strcmp(argv[i], "-E")) {
      stop_stage = 1;
      user_compile = space_merge(user_compile, xstrdup("/E"));
    } else if (!strcmp(argv[i], "-c")) {
      stop_stage = 2;
      user_compile = space_merge(user_compile, xstrdup("/c"));
    } else if (!strcmp(argv[i], "-Wall")) {
      user_compile = space_merge(user_compile, xstrdup("/W3"));
    } else if (!strcmp(argv[i], "-O2")) {
      user_compile = space_merge(user_compile, xstrdup("/Ox"));
    } else if (!strncmp(argv[i], "-Wl", 3)) {
      // do nothing
    } else if (!strcmp(argv[i], "-Wno-unused")) {
      // do nothing
    } else if (!strcmp(argv[i], "-mno-cygwin")) {
      // do nothing
    } else if (!strcmp(argv[i], "-s")) {
      // do nothing
    } else if (!strncmp(argv[i], "-", 1)) {
      fprintf(stderr, "vcw: unknown option `%s'\n",
              argv[i]);
      user_compile = space_merge(user_compile, xstrdup(argv[i]));
    } else {
      number_of_files++;
      path = fix_path(argv[i]);
      user_compile = space_merge(user_compile, path);
    }
  }

  if (!number_of_files) {
    fprintf(stderr, "vcw: No input files\n");
    exit(0);
  }

  if (lib_mode) {
    if (!output_name) {
      fprintf(stderr,"vcw: no library name specified\n");
      exit(1);
    }
    suffix_compile = space_merge(suffix_compile, xstrdup("/out:"));
    suffix_compile = xstrmerge1(suffix_compile, output_name);
  } else {
    if (output_name) {
      if (stop_stage == 1) {
        // stop after preprocessing
        // FIXME!!!
        fprintf(stderr, "vcw: -E and -o options not yet supported\n");
        exit(1);
      } else if (stop_stage == 2) {
        // stop after compilation
        user_compile = space_merge(user_compile, xstrdup("/Fo"));
        user_compile = xstrmerge0(user_compile, output_name);
      } else if (stop_stage == 0) {
        user_compile = space_merge(user_compile, xstrdup("/Fe"));
        user_compile = xstrmerge0(user_compile, output_name);
      }
    }
    if (!output_name && !stop_stage) {
      suffix_compile = space_merge(suffix_compile, xstrdup("/Fe"));
      suffix_compile = xstrmerge0(suffix_compile, strdup("a.exe"));
    }
  }

  if (!lib_mode) {
    // default libraries to link: crtdll user32 kernel32 advapi32 shell32
    if (!stop_stage) {
      // linking required
      suffix_link = space_merge(suffix_link,
                                xstrdup("USER32.LIB KERNEL32.LIB ADVAPI32.LIB SHELL32.LIB /subsystem:console"));
      suffix_compile = space_merge(suffix_compile, xstrdup("/MD"));
    }
    
    /*
    if (libpath && *libpath) {
      prefix_link = space_merge(prefix_link, "/LIBPATH:");
      prefix_link = xstrmerge0(prefix_link, libpath);
    }
    */
  }

  // create command line
  if (lib_mode) {
    cmdline = xstrmerge2("\"", MSVCDir);
    cmdline = xstrmerge1(cmdline, "\\Bin\\LIB.EXE\" /nologo /subsystem:console");
  } else {
    cmdline = xstrmerge2("\"", MSVCDir);
    cmdline = xstrmerge1(cmdline, "\\Bin\\CL.EXE\" /nologo");
  }
  cmdline = space_merge(cmdline, prefix_compile);
  cmdline = space_merge(cmdline, user_compile);
  cmdline = space_merge(cmdline, suffix_compile);
  if (prefix_link || user_link || suffix_link) {
    cmdline = space_merge(cmdline, "/link");
  }
  cmdline = space_merge(cmdline, prefix_link);
  cmdline = space_merge(cmdline, user_link);
  cmdline = space_merge(cmdline, suffix_link);

  // setup environment
  if ((path = getenv("INCLUDE"))) {
    sprintf(env_include, "INCLUDE=%s;%s\\INCLUDE",
            path, MSVCDir);
  } else {
    sprintf(env_include, "INCLUDE=%s\\INCLUDE", MSVCDir);
  }
  _putenv(env_include);
  //fprintf(stderr, "New INCLUDE=%s\n", getenv("INCLUDE"));

  if ((path = getenv("LIB"))) {
    sprintf(env_lib, "LIB=%s;%s\\LIB", path, MSVCDir);
  } else {
    sprintf(env_lib, "LIB=%s\\LIB", MSVCDir);
  }
  _putenv(env_lib);
  //fprintf(stderr, "New LIB=%s\n", getenv("LIB"));

  path = getenv("PATH");
  if (!strcmp(VCOSDir, "WINNT")) {
  } else {
    // WIN95 case
  }
  sprintf(env_path, "PATH=%s\\BIN;%s\\BIN;%s\\TOOLS\\%s;%s\\TOOLS;%s",
          MSDevDir, MSVCDir, VSCommonDir, VCOSDir, VSCommonDir, path);
  _putenv(env_path);
  //fprintf(stderr, "PATH=%s\n", getenv("PATH"));

  fprintf(stderr, "%s\n", cmdline);
  return do_system(cmdline);
}

/* ======================= XALLOC.C STARTS HERE ======================= */
/**
 * NAME:    out_of_mem
 * PURPOSE: report out of virtual memory condition
 */
  static void
out_of_mem(void)
{
  fputs("Failed to allocate more memory!\n", stderr);
  abort();
}

/**
 * NAME:    null_size
 * PURPOSE: report 0 size allocation error
 */
  static void
null_size(void)
{
  fputs("Null size allocation requested!\n", stderr);
  abort();
}

/**
 * NAME:    xmalloc
 * PURPOSE: wrapper over malloc function call
 * NOTE:    xmalloc never returns NULL
 */
  void *
xmalloc(size_t size)
{
  void *ptr;
  if (size == 0) null_size();
  ptr = malloc(size);
  if (ptr == NULL) out_of_mem();
  return ptr;
}

/**
 * NAME:    xcalloc
 * PURPOSE: wrapper over calloc function
 * NOTE:    xcalloc never returns NULL
 */
  void *
xcalloc(size_t nitems, size_t elsize)
{
  void *ptr;
  if (nitems == 0 || elsize == 0) null_size();
  ptr = calloc(nitems, elsize);
  if (ptr == NULL) out_of_mem();
  return ptr;
}

/**
 * NAME:    xfree
 * PURPOSE: wrapper over free function
 * NOTE:    accepts NULL pointer as argument
 */
  void
xfree(void *ptr)
{
  if (ptr == NULL) return;
  free(ptr);
}

/**
 * NAME:    xrealloc
 * PURPOSE: wrapper over realloc function
 * NOTE:    if ptr == NULL,  realloc = malloc
 *          if size == NULL, realloc = free
 *          if ptr == NULL && size == NULL, ?
 */
  void *
xrealloc(void *ptr, size_t size)
{
  if (ptr == NULL && size == 0) null_size();
  ptr = realloc(ptr,size);
  if (ptr == NULL) out_of_mem();
  return ptr;
}

/**
 * NAME:    xstrdup
 * PURPOSE: wrapper over strdup function
 * NOTE:    strdup(NULL) returns ""
 */
  char *
xstrdup(char const*str)
{
  char *ptr;
  if (str == NULL) str = "";
  ptr = strdup(str);
  if (ptr == NULL) out_of_mem();
  return ptr;
}

/**
 * NAME:    xmemdup
 * PURPOSE: returns a copy of the string in the heap
 * ARGS:    str  - string to copy (might not be \0 terminated)
 *          size - string length
 * RETURN:  copy of the string str with \0 terminator added
 */
  char *
xmemdup (char const *str, size_t size)
{
  char *ptr;
  if (str == NULL) str = "";
  ptr = xmalloc (size + 1);
  if (ptr == NULL) out_of_mem();
  memcpy (ptr, str, size);
  ptr[size] = 0;
  return ptr;
}

/**
 * NAME:    xstrmerge0
 * PURPOSE: concatenate two strings
 * ARGS:    str1 - string 1
 *          str2 - string 2
 * RETURN:  concatenation of two strings (allocated in heap)
 * NOTE:    str1 and str2 are freed via xfree call after concatenation
 */
  char *
xstrmerge0(char *str1, char *str2)
{
  char *res;

  if (str1 == NULL && str2 == NULL)
    {
      return NULL;
    }

  if (str1 == NULL)
    {
      return str2;
    }

  if (str2 == NULL)
    {
      return str1;
    }

  res = (char*) xmalloc(strlen(str1) + strlen(str2) + 1);
  strcpy(res, str1);
  strcat(res, str2);
  xfree(str1);
  xfree(str2);
  return res;
}

/**
 * NAME:    xstrmerge1
 * PURPOSE: concatenate two strings
 * ARGS:    str1 - string 1
 *          str2 - string 2
 * RETURN:  concatenation of two strings
 * NOTE:    str1 - freed after concatenation
 *          str2 - not freed
 */
  char *
xstrmerge1(char *str1, char const *str2)
{
  char *res;

  if (str1 == NULL && str2 == NULL)
    return NULL;

  if (str1 == NULL)
    return xstrdup(str2);

  if (str2 == NULL)
    return str1;

  res = (char*) xmalloc(strlen(str1) + strlen(str2) + 1);
  strcpy(res, str1);
  strcat(res, str2);
  xfree(str1);
  return res;
}

/**
 * NAME:    xstrmerge2
 * PURPOSE: concatenate two strings
 * ARGS:    str1 - string 1
 *          str2 - string 2
 * RETURN:  concatenation of two strings
 * NOTE:    str1 - not freed after concatenation
 *          str2 - not freed
 */
  char *
xstrmerge2(char const *str1, char const *str2)
{
  char *res;

  if (str1 == NULL && str2 == NULL)
    return NULL;

  if (str1 == NULL)
    return xstrdup(str2);

  if (str2 == NULL)
    return xstrdup(str1);

  res = (char*) xmalloc(strlen(str1) + strlen(str2) + 1);
  strcpy(res, str1);
  strcat(res, str2);
  return res;
}

/**
 * NAME:    xstrmerge3
 * PURPOSE: concatenate two strings
 * ARGS:    str1 - string 1
 *          str2 - string 2
 * RETURN:  concatenation of two strings
 * NOTE:    str1 - not freed
 *          str2 - freed after concatenation
 */
  char *
xstrmerge3(char const *str1, char *str2)
{
  char *res;

  if (!str1 && !str2) return 0;
  if (!str1)          return str2;
  if (!str2)          return xstrdup(str1);

  res = (char*) xmalloc(strlen(str1) + strlen(str2) + 1);
  strcpy(res, str1);
  strcat(res, str2);
  xfree(str2);
  return res;
}

/**
 * NAME:    xexpand
 * PURPOSE: expand expandable array of strings
 * ARGS:    arr - pointer to expandable array structure
 */
  void
xexpand(strarray_t *arr)
{
  if (arr->u < arr->a) return;

  if (!arr->a)
    {
      arr->a = 32;
      arr->v = (char**) xcalloc(arr->a, sizeof(char **));
      return;
    }

  arr->v = (char**) xrealloc(arr->v, arr->a * sizeof(char**) * 2);
  memset(arr->v + arr->a, 0, arr->a * sizeof(char**));
  arr->a *= 2;
}

/**
 * NAME:    xexpand2
 * PURPOSE: expand generic expandable array
 * ARGS:    arr    - pointer to expandable array structure
 *          elsize - size of an element of the array
 */
  void
xexpand2(arr, elsize)
     genarray_t  *arr;
     size_t       elsize;
{
  if (!arr) return;

  if (elsize <= 0) elsize = sizeof(int);
  if (arr->u < arr->a) return;

  if (!arr->a)
    {
      arr->a = 32;
      arr->v = xcalloc(arr->a, elsize);
      return;
    }

  arr->v = (void*) xrealloc(arr->v, arr->a * elsize * 2);
  memset((char*) arr->v + arr->a * elsize, 0, arr->a * elsize);
  arr->a *= 2;
}

/**
 * NAME:    xexpand3
 * PURPOSE: unconditionally expand the array
 * ARGS:    arr    - array to expand
 *          elsize - element size
 */
  void
xexpand3(arr, elsize)
     genarray_t  *arr;
     size_t       elsize;
{
  if (!arr) return;

  if (elsize <= 0) elsize = sizeof(int);

  if (!arr->a)
    {
      arr->a = 32;
      arr->v = xcalloc(arr->a, elsize);
      return;
    }

  arr->v = (void*) xrealloc(arr->v, arr->a * elsize * 2);
  memset((char*) arr->v + arr->a * elsize, 0, arr->a * elsize);
  arr->a *= 2;
}

/**
 * NAME:    xexpand4
 * PURPOSE: unconditionally expand the array
 * ARGS:    arr     - array to expand
 *          elsize  - element size
 *          newsize - new size of the array
 */
  void
xexpand4(arr, elsize, newsize)
     genarray_t *arr;
     size_t      elsize;
     int         newsize;
{
  int newsz;

  if (!arr) return;
  if (newsize <= arr->a) return;

  if (elsize <= 0) elsize = sizeof(int);
  newsz = arr->a;
  if (!newsz) newsz = 32;
  while (newsz < newsize)
    newsz *= 2;

  arr->v = (void*) xrealloc(arr->v, newsz * elsize);
  memset((char*) arr->v + arr->a * elsize, 0, (newsz - arr->a) * elsize);
  arr->a = newsz;
}

  void
xstrarrayfree(strarray_t *a)
{
  int i;

  if (!a) return;

  for (i = 0; i < a->u; i++) {
    xfree(a->v[i]);
  }
  xfree(a->v);
  a->u = a->a = 0;
  a->v = 0;
}

/* ======================= XALLOC.C ENDS HERE ======================= */

  char *
os_DirName(char const *str)
{
  int         len;
  char const *s;

  if (!str) return xstrdup("");
  len = strlen(str);
  for (s = str + len - 1; s >= str; s--) {
    if (*s == '/' || *s == '\\' || *s == ':') {
      break;
    }
  }
  if (s < str) {
    return xstrdup(".");
  }
  if (*s == ':') s++;

  return xmemdup(str, s - str);
}

/**
 * Local variables:
 *  compile-command: "cl-proxy.exe -O2 -Wall vcw.c -o a.exe"
 * End:
 */
